//
// Copyright 2017 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//

package de.frankmuenster.jameica.depot.finanzen;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.FfmStockInfoBean;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.FfmWertpapierHistory;
import de.frankmuenster.jameica.depot.finanzen.quandl.QuandlWertpapierHistory;
import de.willuhn.logging.Logger;

/**
 *
 * @author Frank Mnster
 *
 */
public class WertpapierHistory {

	/**
	 * DatenAbfrage erfolgreich
	 */
	protected transient boolean dataRequestOK;

	/**
	 * Anfangsdatum der Werte
	 */
	protected transient Date fromDate;

	/**
	 * Das Yahoo Symbol
	 */
	protected transient String symbol;

	/**
	 * Der Quandl Code
	 */
	protected transient String quandlCode;

	/**
	 * Der StockInfoBean fr die Brese Frankfurt
	 */
	protected transient FfmStockInfoBean ffmInfoBean;

	/**
	 * Enddatum der Werte
	 */
	protected transient Date toDate;

	/**
	 * Datenquelle
	 */
	private transient String dataSource;

	/**
	 * Liefert eine Liste mit den historischen Werten des Wertpapiers
	 */
	protected transient List<HistoryValueBean> wphv = new ArrayList<HistoryValueBean>();

	/**
	 * Konstruktor mit Angabe eines Wertpapier-Okjektes
	 */
	public WertpapierHistory(final Wertpapier wertpapier) throws FinanceException {

		String symbol = null;
		try {
			symbol = wertpapier.getSymbol();
		} catch (RemoteException e) { // NOPMD
			// Hier machen wir nix
		}
		String quandlCode = null;
		try {
			quandlCode = wertpapier.getQuandlCode();
		} catch (RemoteException e) { // NOPMD
			// Und hier auch nix
		}
		FfmStockInfoBean ffmInfoBean = null;
		try {
			ffmInfoBean = new FfmStockInfoBean(wertpapier.getFfmInstrument(), wertpapier.getFfmName(), null, null);
		} catch (RemoteException e) { // NOPMD
			// Und hier auch nix
		}

		// Und nun die eigentliche Arbeit
		this.init(symbol, quandlCode, ffmInfoBean);

	}

	/**
	 * Konstruktor mit Angabe des Yahoo-Symbols, des Quandl-Codes und der FFM
	 * Brsenname
	 *
	 * @param quandlCode
	 * @param ffmName
	 *
	 * @throws FinanceException
	 */
	public WertpapierHistory(final String symbol, final String quandlCode, final FfmStockInfoBean ffmInfoBean)
			throws FinanceException {
		// In privater Methode, damit sie vom 2.ten Konstruktor mit verwendet
		// werden kann.
		this.init(symbol, quandlCode, ffmInfoBean);
	}

	/**
	 * @param symbol
	 * @param quandlCode
	 * @param ffmInfoBean
	 * @throws FinanceException
	 */
	private void init(final String symbol, final String quandlCode, final FfmStockInfoBean ffmInfoBean)
			throws FinanceException {

		this.symbol = symbol;
		this.quandlCode = quandlCode;
		this.ffmInfoBean = ffmInfoBean;
		this.dataRequestOK = false;
		this.dataSource = StringUtils.EMPTY;

		// Mal bei Quandl versuchen
		if (StringUtils.isNotEmpty(quandlCode)) {
			try {
				final QuandlWertpapierHistory qwh = new QuandlWertpapierHistory(quandlCode, Settings.getQuandlApiKey());
				this.wphv = qwh.loadData();
				this.dataSource = "Quandl: " + quandlCode; //$NON-NLS-1$
			} catch (FinanceException ex) {
				// Hier nur loggen, wir haben ja noch die Brse FFM
				Logger.error(ex.getLocalizedMessage());
			}
		}

		if (this.wphv.isEmpty() && ffmInfoBean != null) {
			try {
				// probieren wir es mal mit der Brse FFM
				final FfmWertpapierHistory fwh = new FfmWertpapierHistory(ffmInfoBean);
				this.wphv = fwh.loadData();
				this.dataSource = "Brse Frankfurt"; //$NON-NLS-1$
			} catch (FinanceException ex) {
				// Nun werfen wir die Exception doch
				Logger.error(ex.getLocalizedMessage());
				this.dataSource = StringUtils.EMPTY;
				throw ex;
			}
		}

		if (this.wphv.isEmpty()) {
			this.dataSource = StringUtils.EMPTY;
			throw new FinanceException("Keine historischen Daten gefunden", FinanceExceptionType.DATA_NOT_FOUND); //$NON-NLS-1$
		}
		this.dataRequestOK = true;

		// Dann sortieren.
		Collections.sort(this.wphv);

		// und das Anfangs- und Enddatum setzten
		this.fromDate = this.wphv.get(0).getDate();
		this.toDate = this.wphv.get(this.wphv.size() - 1).getDate();
	}

	/**
	 * Liefert das Anfangsdatum
	 *
	 * @return fromDate
	 */
	public Date getFromDate() {
		return this.fromDate;
	}

	/**
	 * Liefert eine Liste mit den historischen Werten des Wertpapiers.
	 *
	 * @return wphv Liste mit den HistoryValueBean
	 */
	public List<HistoryValueBean> getHistoryValues() {
		return this.wphv;
	}

	/**
	 * Liefert ddas Yahoo Symbol
	 *
	 * @return symbol
	 */
	public String getSymbol() {
		return this.symbol;
	}

	/**
	 * Liefert die historischen Werte f&uuml;r eine angegeben Zeitraum.
	 *
	 * @param fromDate
	 *            das Anfangsdatum
	 * @param toDate
	 *            das Enddatum
	 * @return Die Liste mit HistoryValueBean
	 */
	public List<HistoryValueBean> getTimeRangeValues(final Date fromDate, final Date toDate) {

		final List<HistoryValueBean> rangeValues = new ArrayList<HistoryValueBean>();

		for (final HistoryValueBean value : this.wphv) {
			final long dateInt = value.getDate().getTime();
			if (dateInt > fromDate.getTime() && dateInt < toDate.getTime()) {
				rangeValues.add(value);
			}
		}

		return rangeValues;
	}

	/**
	 * Liefert die historischen Werte f&uuml;r ein angegebenes Datum. Falls kein
	 * Daten f&uuml; das angegeben Datum zur Verf&uuml;gung stehen, wird der
	 * n&auml;chste davorliegende Handelstag genommen. Liegt kein Tage vor
	 * DATUM, werden die Daten des ersten zur Verf&uuml;gung stehenden Tages
	 * verwendet. Liegen gar keine Daten vor wir NULL zur&uuml;ckgegeben.
	 *
	 * @param date
	 *            das Datum
	 * @return Der HistoryValueBean
	 */
	public HistoryValueBean getValuesAtDate(final Date date) {

		HistoryValueBean returnValue = null;

		if (!this.wphv.isEmpty()) {
			returnValue = this.wphv.get(0);

			for (final HistoryValueBean value : this.wphv) {

				final Date dateValue = value.getDate();
				if (DateUtils.isSameDay(date, dateValue)) {
					// Gesuchtes Datum gefunden
					returnValue = value;
					break;
				} else if (date.before(dateValue)) {
					// Wir sind am gesuchten Datum vorbei
					break;
				}
				returnValue = value;
			}
		}
		return returnValue;
	}

	/**
	 * Liefert das Enddatum
	 *
	 * @return toDate
	 */
	public Date getToDate() {
		return this.toDate;
	}

	/**
	 * War der Datenabruf OK.
	 *
	 * @return the dataRequestOK
	 */
	public Boolean isDataRequestOK() {
		return this.dataRequestOK;
	}

	/**
	 * Liefert die Datenquelle
	 *
	 * @return Datenquelle (Yahoo, Brese FFM)
	 */
	public String getDataSource() {
		return this.dataSource;
	}

}