/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.parts;

import java.rmi.RemoteException;
import java.sql.Time;
import java.util.Date;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.gui.controller.WertpapierDetailController;
import de.frankmuenster.jameica.depot.gui.input.LabelInputExtended;
import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.util.Color;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Liefert ein Part, das Details zu einem Wertpapier anzeigt. Dazu geh&ouml;ren
 * der Name, das Yahoo-Symbol, der FfmName, der Quandl-Code, die Anzahl, das
 * letzte Abfragedatum und der Kurs, sowie die Gesamtsumme des Papieres im
 * Depot. Ist das Papier nicht bei Yahoo gelistet, knnen Datum, Kurs und
 * W&auml;hrung gesetzt werden.
 *
 * @author Frank Mnster
 *
 */
public class WertpapierDetailsPart implements Part {

	/**
	 * Veraltertes Handelsdatum in Millisekunden.
	 */
	private final static long OLD_TRADE_DAYS = 7 * 24 * 60 * 60 * 1000;

	/**
	 * Veraltertes Handelsdatum Anzeigentext
	 */
	private final static String OLD_TRADE_TEXT = Settings.getI18n().tr("WertpapierDetailsPart.toOldLastTrade"); //$NON-NLS-1$

	/**
	 * Pfeil nach oben Zeichen
	 */
	private final static char ARROW_UP = '\u2191';

	/**
	 * Pfeil nach unten Zeichen
	 */
	private final static char ARROW_DOWN = '\u2193';

	/**
	 * WertpapierDetail-Kontroller, damit auch Eingabefelder verwendet werden
	 * k&ouml;nnen.
	 */
	private final transient WertpapierDetailController control;

	/**
	 * Das Wertpapier
	 */
	private final transient Wertpapier wertpapier;

	/**
	 * Der WP-Name
	 */
	private transient LabelInputExtended nameLbl;

	/**
	 * Das Yahoo-Symbol
	 */
	private transient LabelInputExtended symbolLbl;

	/**
	 * Der Brse FFM Name
	 */
	private transient LabelInputExtended ffmNameLbl;

	/**
	 * Der Qunadl Code
	 */
	private transient LabelInputExtended quandlCodeLbl;

	/**
	 * Die WP-St&uuml;ckanzahl
	 */
	private transient LabelInputExtended anzahlLbl;

	/**
	 * Das letzte Handelsdatum
	 */
	private transient LabelInputExtended datumLbl;

	/**
	 * Der letzte Handelswert
	 */
	private transient LabelInputExtended letzterWertLbl;

	/**
	 * Der Gesamtwert
	 */
	private transient LabelInputExtended totalLbl;

	/**
	 * Erstellt ein neues WertpapierDetailPart.
	 *
	 * @param wertpapier
	 *            das Papier dessen Details angezeigt werden sollen.
	 * @param control
	 *            der WertpapierDeatils-View Kontrolleur
	 */
	public WertpapierDetailsPart(final Wertpapier wertpapier, final WertpapierDetailController control) {
		this.control = control;
		this.wertpapier = wertpapier;
	}

	/**
	 * @see Part#paint(Composite)
	 */
	@Override
	public void paint(final Composite parent) throws RemoteException {

		final SimpleContainer container = new SimpleContainer(parent, false, 8);

		if (this.wertpapier == null) {
			container.addText(Settings.getI18n().tr("WertpapierDetailsPart.noStockInfoAvailable"), //$NON-NLS-1$
					true);
		} else {
			this.nameLbl = new LabelInputExtended(this.wertpapier.getName());
			container.addLabelPair(Settings.getI18n().tr("WertpapierDetailsPart.lblName"), this.nameLbl); //$NON-NLS-1$

			this.symbolLbl = new LabelInputExtended(this.wertpapier.getSymbol());
			container.addLabelPair(Settings.getI18n().tr("WertpapierDetailsPart.lblSymbole"), this.symbolLbl); //$NON-NLS-1$

			this.ffmNameLbl = new LabelInputExtended(this.wertpapier.getFfmName());
			this.ffmNameLbl.setToolTip(this.wertpapier.getFfmInstrument());
			container.addLabelPair(Settings.getI18n().tr("WertpapierDetailsPart.lblFfmName"), this.ffmNameLbl); //$NON-NLS-1$

			this.quandlCodeLbl = new LabelInputExtended(this.wertpapier.getQuandlCode());
			container.addLabelPair(Settings.getI18n().tr("WertpapierDetailsPart.lblQuandle"), this.quandlCodeLbl); //$NON-NLS-1$

			final double anzahl = this.wertpapier.getAnzahl();
			this.anzahlLbl = new LabelInputExtended(FormatterUtils.DECIMAL_FORMAT_6.get().format(anzahl),
					Settings.getI18n().tr("WertpapierDetailsPart.lblNumberStocks")); //$NON-NLS-1$
			// container.addLabelPair(Settings.getI18n().tr("WertpapierDetailsPart.lblCount"),
			// this.anzahlLbl); //$NON-NLS-1$

			if (this.wertpapier.isInYahoo()) {
				this.wpInYahoo(container);
			} else {
				this.wpNotInYahoo(parent);
			}

		}

	}

	/**
	 * Aktualisiert die Anzeige.
	 */
	public void update() {

		try {

			this.nameLbl.setValue(this.wertpapier.getName());
			this.symbolLbl.setValue(this.wertpapier.getSymbol());
			this.anzahlLbl.setValue(FormatterUtils.DECIMAL_FORMAT_6.get().format(this.wertpapier.getAnzahl()));

			// Datum und LetzerHandelsWert nur wenn nicht als Inputs
			if (this.datumLbl != null) {
				this.datumLbl
						.setValue(FormatterUtils.GERMAN_DATE_FORMAT.get().format(this.wertpapier.getLastTradeDate()));
				final Date date = this.wertpapier.getLastTradeDate();
				final Time time = this.wertpapier.getLastTradeTime();

				if (DateUtils.dateDiff(date) > WertpapierDetailsPart.OLD_TRADE_DAYS) {
					this.datumLbl.setColor(Color.ERROR);
					this.datumLbl.setToolTip(Settings.getI18n().tr(WertpapierDetailsPart.OLD_TRADE_TEXT));
					Application.getMessagingFactory().sendMessage(new StatusBarMessage(
							Settings.getI18n().tr(WertpapierDetailsPart.OLD_TRADE_TEXT), StatusBarMessage.TYPE_INFO));
				} else {
					this.datumLbl.setColor(Color.FOREGROUND);
					if (time != null) {
						this.datumLbl.setToolTip(FormatterUtils.SIMPLE_TIME_FORMAT.get().format(time));
					}
				}
			}
			if (this.letzterWertLbl != null) {
				this.letzterWertLbl.setValue(
						FormatterUtils.DECIMAL_FORMAT_3.get().format(this.wertpapier.getLastTradePriceOnly()) + " " //$NON-NLS-1$
								+ this.wertpapier.getCurrency());
			}

			final StringBuffer change = new StringBuffer(
					FormatterUtils.DECIMAL_FORMAT.get().format(this.wertpapier.getChangeInPercentage()));
			if (this.wertpapier.getChangeInPercentage() < 0) {
				change.append(WertpapierDetailsPart.ARROW_DOWN).append(change);
				this.letzterWertLbl.setColor(Color.ERROR);
			} else {
				change.append(WertpapierDetailsPart.ARROW_UP).append(change);
				this.letzterWertLbl.setColor(Color.FOREGROUND);
			}
			this.letzterWertLbl.setComment(change + "%"); //$NON-NLS-1$
			this.letzterWertLbl.setToolTip(Settings.getI18n().tr("WertpapierDetailsPart.lblValueTooltip")); //$NON-NLS-1$

			this.totalLbl.setValue(FormatterUtils.CURRENCY_FORMAT.get()
					.format(this.wertpapier.getAnzahl() * this.wertpapier.getLastTradePriceOnly()) + " " //$NON-NLS-1$
					+ this.wertpapier.getCurrency());
		} catch (RemoteException e) {
			Logger.error(Settings.getI18n().tr("WertpapierDetailsPart.refreshStockDeatilPartError") //$NON-NLS-1$
					+ e.getMessage());
			Application.getMessagingFactory()
					.sendMessage(new StatusBarMessage(
							Settings.getI18n().tr("WertpapierDetailsPart.refreshStockDeatilPartErrorMsg"), //$NON-NLS-1$
							StatusBarMessage.TYPE_ERROR));
		}
	}

	/**
	 * Zeichnet die WP Daten aus Yahoo.
	 *
	 * @param container
	 *            der schon erstellte Container.
	 * @param anzahlLbl2
	 * @throws RemoteException
	 */
	private void wpInYahoo(final Container container) throws RemoteException {

		final Date date = this.wertpapier.getLastTradeDate();
		final Time time = this.wertpapier.getLastTradeTime();
		if (date == null) {
			this.datumLbl = new LabelInputExtended(StringUtils.EMPTY,
					Settings.getI18n().tr("WertpapierDetailsPart.lblDatePleaseRefesh")); //$NON-NLS-1$
		} else {
			this.datumLbl = new LabelInputExtended(FormatterUtils.SIMPLE_TIME_FORMAT.get().format(date));
			// long dateDiff = 7 * 24 * 60 * 60 * 1000;
			if (DateUtils.dateDiff(date) > WertpapierDetailsPart.OLD_TRADE_DAYS) {
				this.datumLbl.setColor(Color.ERROR);
				this.datumLbl.setToolTip(Settings.getI18n().tr(WertpapierDetailsPart.OLD_TRADE_TEXT));
				Application.getMessagingFactory().sendMessage(new StatusBarMessage(
						Settings.getI18n().tr(WertpapierDetailsPart.OLD_TRADE_TEXT), StatusBarMessage.TYPE_INFO));
			} else {
				// Uhrzeit anzeigen
				if (time != null) {
					this.datumLbl.setToolTip(FormatterUtils.SIMPLE_TIME_FORMAT.get().format(time));
				}
			}
		}
		container.addLabelPair(Settings.getI18n().tr("WertpapierDetailsPart.lblDate"), this.datumLbl); //$NON-NLS-1$

		final double lastTradePrice = this.wertpapier.getLastTradePriceOnly();
		this.letzterWertLbl = new LabelInputExtended(
				FormatterUtils.DECIMAL_FORMAT_3.get().format(lastTradePrice) + " " + this.wertpapier.getCurrency()); //$NON-NLS-1$

		// Platz machen fr die nderungsanzeige
		this.letzterWertLbl.setComment(StringUtils.EMPTY);

		container.addLabelPair(Settings.getI18n().tr("WertpapierDetailsPart.lblValue"), this.letzterWertLbl); //$NON-NLS-1$

		// Anzahl hinzufgen
		container.addLabelPair(Settings.getI18n().tr("WertpapierDetailsPart.lblCount"), this.anzahlLbl); //$NON-NLS-1$

		// Gesammtsumme
		final double anzahl = this.wertpapier.getAnzahl() * this.wertpapier.getLastTradePriceOnly();
		this.totalLbl = new LabelInputExtended(
				FormatterUtils.CURRENCY_FORMAT.get().format(anzahl) + " " + this.wertpapier.getCurrency(), Settings //$NON-NLS-1$
						.getI18n().tr("WertpapierDetailsPart.lblTotalTooltip")); //$NON-NLS-1$
		container.addLabelPair(Settings.getI18n().tr("WertpapierDetailsPart.lblTotal"), this.totalLbl); //$NON-NLS-1$

	}

	/**
	 * Zeichnet die WP Daten aus Yahoo.
	 *
	 * @param parent
	 * @param anzahlLbl
	 * @throws RemoteException
	 */
	private void wpNotInYahoo(final Composite parent) throws RemoteException {
		final SimpleContainer notYahoo = new SimpleContainer(parent, false, 10);

		notYahoo.addLabelPair(Settings.getI18n().tr("WertpapierDetailsPart.lblDate"), //$NON-NLS-1$
				this.control.getDatum());
		notYahoo.addLabelPair(Settings.getI18n().tr("WertpapierDetailsPart.lblValue"), //$NON-NLS-1$
				this.control.getLetzterKursWert());
		notYahoo.addPart(this.control.getWaehrung());

		// Anzahl hinzufgen
		notYahoo.addLabelPair(Settings.getI18n().tr("WertpapierDetailsPart.lblCount"), this.anzahlLbl); //$NON-NLS-1$

		// Gesamtsumme
		final double totalValue = this.wertpapier.getAnzahl() * this.wertpapier.getLastTradePriceOnly();
		this.totalLbl = new LabelInputExtended(
				FormatterUtils.CURRENCY_FORMAT.get().format(totalValue) + " " + this.wertpapier.getCurrency(), Settings //$NON-NLS-1$
						.getI18n().tr("WertpapierDetailsPart.lblTotalTooltip")); //$NON-NLS-1$
		notYahoo.addLabelPair(Settings.getI18n().tr("WertpapierDetailsPart.lblTotal"), this.totalLbl); //$NON-NLS-1$

		final StringBuffer message = new StringBuffer(Settings.getI18n().tr("WertpapierDetailsPart.quoteNotinYahoo")); //$NON-NLS-1$

		if (this.control.getDatum().getValue() != null && DateUtils
				.dateDiff((Date) this.control.getDatum().getValue()) > WertpapierDetailsPart.OLD_TRADE_DAYS) {
			message.append(" | ").append(Settings.getI18n().tr(WertpapierDetailsPart.OLD_TRADE_TEXT)); //$NON-NLS-1$
		}

		Application.getMessagingFactory()
				.sendMessage(new StatusBarMessage(message.toString(), StatusBarMessage.TYPE_INFO));
	}

}
